<?php
namespace Foundation\Form;
/**
 * Element interface
 * 
 * @package Foundation\form\element
 */
interface Element
{

  /**
   * Add a message to the element
   * @param string $text
   */
  function addMessage($text);

  /**
   * Get Messages
   * @return array
   */
  function getMessages();

  /**
   * Add a validator
   * @param \Foundation\Form\Validator
   */
  function addValidator(Validator $validator);

  /**
   * Prepend a validator to any onthe stack already
   * @param \Foundation\Form\Validator
   */
  function prependValidator(Validator $validator);

  /**
   * Add a filter
   * @param \Foundation\Form\Filter
   */
  function addFilter(Filter $filter);

  /**
   * Prepend a filter to any other ones
   * @param \Foundation\Form\Filter
   */
  function prependFilter(Filter $filter);

  /**
   * Run any validator preRender methods
   */
  public function preRender();

  /**
   * Process Input
   * 
   * @param \Foundation\Form\Input $input
   * @return boolean
   */
  function processInput(\Foundation\Form\Input $input);

  /**
   * Set the name
   * @param string $name
   */
  function setName($name);

  /**
   * Get the name
   * @return string $name
   */
  function getName();

  /**
   * Set the value
   * @param string $value
   */
  function setValue($value);

  /**
   * Get the value
   * @return string $value
   */
  function getValue();

  /**
   * Set the defaultvalue
   * @param string $value
   */
  function setDefaultValue($value);

  /**
   * Get the default value
   * @return string $value
   */
  function getDefaultValue();

  /**
   * Set the accesskey
   * @param string $accesskey
   */
  function setAccesskey($accesskey);

  /**
   * Get the accesskey
   * @return string $accesskey
   */
  function getAccesskey();

  /**
   * Set the tabindex
   * @param string $tabindex
   */
  function setTabindex($tabindex);

  /**
   * Get the tabindex
   * @return string $tabindex
   */
  function getTabindex();

  /**
   * Set the instructions
   * @param string $instructions
   */
  function setInstructions($instructions);

  /**
   * Get the instructions
   * @return string $instructions
   */
  function getInstructions();

  /**
   * Set the format
   * @param string $format
   */
  function setFormat($format);

  /**
   * Get the legend
   * @return string $legend
   */
  function getFormat();

  /**
   * Convert the element to an array
   * @return array
   */
  function toArray();

}